ColorConfiguratorPickerDialog = {}

local ColorConfiguratorPickerDialog_mt = Class(ColorConfiguratorPickerDialog, ColorPickerDialog)

function ColorConfiguratorPickerDialog.new(target, custom_mt, colorConfigurator)
	local self = ColorPickerDialog.new(target, custom_mt or ColorConfiguratorPickerDialog_mt)

	self.colorConfigurator = colorConfigurator

	self.buttonPressedTime = 0
	self.buttonPressTime = 250
	self.pressedButton = nil
	self.openColorConfiguratorDialog = false
	self.removeColorButtons = {}
	self.previousColors = {}

	return self
end

function ColorConfiguratorPickerDialog:delete()
	if self.removeColorButtonTemplate ~= nil then
		self.removeColorButtonTemplate:delete()
	end

	ColorConfiguratorPickerDialog:superClass().delete(self)
end

function ColorConfiguratorPickerDialog:onGuiSetupFinished()
	ColorConfiguratorPickerDialog:superClass().onGuiSetupFinished(self)

	local buttonsPC = self.backButton.parent

	local customColorButton = ButtonElement.new(self)

	customColorButton:applyProfile("buttonActivate", true)
	customColorButton:setText(g_i18n:getText("colorConfigurator_button_customColor"))
	customColorButton:setCallback("onClickCallback", "onOpenColorConfiguratorDialog")
	customColorButton:applyScreenAlignment()

	table.insert(buttonsPC.elements, 2, customColorButton)
	customColorButton.parent = buttonsPC

	self.customColorButton = customColorButton

	local removeColorButtonTemplate = ButtonElement.new(self)

	removeColorButtonTemplate:applyProfile("colorConfiguratorRemoveColorButton", true)
	removeColorButtonTemplate:setCallback("onClickCallback", "onClickRemoveColorButton")
	removeColorButtonTemplate:applyScreenAlignment()

	self.dialogElement:addElement(removeColorButtonTemplate)
	self.removeColorButtonTemplate = removeColorButtonTemplate

	self.customConfigIconUVs = GuiUtils.getUVs("583px 7px 22px 22px")
	self.customConfigIconSize = GuiUtils.getNormalizedValues("12px 12px", removeColorButtonTemplate.outputSize)

	self.colorButtonLayout:setSize(unpack(GuiUtils.getNormalizedValues("418px 52px", self.colorButtonLayout.outputSize)))
end

function ColorConfiguratorPickerDialog:onOpenColorConfiguratorDialog(element, button)
	local dialog = g_gui:showDialog("ColorConfiguratorDialog", true)

	if dialog ~= nil and button ~= nil then
		local colorIndex = self.colorMapping[button]
		local config = self.colors[colorIndex]
		local colors = config.color
		local material = Utils.getNoNil(config.material, 0)

		dialog.target:setValues(colors, material, config.material ~= nil)
	end
end

function ColorConfiguratorPickerDialog:setColors(colors, defaultColor, defaultColorMaterial)
	self.previousColors = {colors, defaultColor, defaultColorMaterial}

	for i = #self.removeColorButtons, 1, -1 do
		self.removeColorButtons[i]:delete()
	end

	colors = table.copy(colors, math.huge)

	local customConfigIndex = nil
	local newColors = {}

	for i, color in ipairs(colors) do
		if color.isCustomConfig then
			if customConfigIndex == nil then
				customConfigIndex = i
			end
		else
			table.insert(newColors, color)
		end
	end

	local configName = self.args.configName
	local filename = self.target.storeItem.xmlFilename
	local addConfigs = self.colorConfigurator:getAdditionalConfigs(filename, configName)

	if addConfigs ~= nil then
		for _, addColor in ipairs(addConfigs) do
			table.insert(newColors, addColor)
		end
	end

	local colorIndex = self.target.configurations[configName]

	if colorIndex == customConfigIndex then
		local addCustomConfigButton = true

		if self.target.licensePlateData ~= nil and self.target.licensePlateData.colorConfiguratorData ~= nil then
			local customConfig = self.target.licensePlateData.colorConfiguratorData[configName]

			if customConfig ~= nil then
				local customConfigColor = customConfig.color
				local customConfigMaterial = customConfig.material

				if addConfigs ~= nil then
					for _, addConfig in ipairs(addConfigs) do
						local addConfigColor = addConfig.color
						local addConfigMaterial = addConfig.material

						if addConfigMaterial == customConfigMaterial and table.equals(addConfigColor, customConfigColor) then
							addCustomConfigButton = false
							self.defaultColor = addConfig

							break
						end
					end
				end

				if addCustomConfigButton then
					local customColor = colors[colorIndex]

					customColor.color = customConfigColor
					customColor.material = customConfigMaterial

					table.insert(newColors, customColor)

					self.defaultColor = customColor
				end
			end
		end
	end

	ColorConfiguratorPickerDialog:superClass().setColors(self, newColors, defaultColor, defaultColorMaterial)

	self.removeColorButtons = {}
	self.additionalColorsMapping = {}

	for i, color in ipairs(newColors) do
		if color.isSelectable ~= false and color.isAddConfig or color.isCustomConfig then
			local removeColorButton = self.removeColorButtonTemplate:clone(self.dialogElement)

			local absPosition = self.buttonMapping[color].absPosition
			local size = self.buttonMapping[color].size
			local baseSize = removeColorButton.size

			local x, y = absPosition[1] + size[1] - baseSize[1], absPosition[2] + size[2] - baseSize[2]

			removeColorButton:setAbsolutePosition(x, y)
			removeColorButton:setVisible(true)

			if color.isCustomConfig then
				removeColorButton:setIconSize(unpack(self.customConfigIconSize))
				removeColorButton:setImageUVs(_, self.customConfigIconUVs)
				removeColorButton:setDisabled(true)
			end

			table.insert(self.removeColorButtons, removeColorButton)
			self.additionalColorsMapping[removeColorButton] = i
		end
	end

	self:updateBlurArea()
end

function ColorConfiguratorPickerDialog:onClickColorButton(element)
	local colorIndex = self.colorMapping[element]
	local selectedColor = self.colors[colorIndex]

	if selectedColor ~= nil and selectedColor.isAddConfig or selectedColor.isCustomConfig then
		self:onClickBack()

		local color = selectedColor.color
		local material = selectedColor.material
		local colors = {color[1], color[2], color[3], material}

		self.colorConfigurator:setColor(colors)
	else
		self:sendCallback(colorIndex)
	end
end

function ColorConfiguratorPickerDialog:update(dt)
	ColorConfiguratorPickerDialog:superClass().update(self, dt)

	if self.colors ~= nil then
		for i, color in ipairs(self.colors) do
			local button = self.buttonMapping[color]

			if button ~= nil then
				if button:getOverlayState() == GuiOverlay.STATE_PRESSED then
					self.pressedButton = button
				end
			end
		end

		if self.pressedButton ~= nil then
			if self.pressedButton:getOverlayState() == GuiOverlay.STATE_PRESSED then
				self.buttonPressedTime = self.buttonPressedTime + dt

				if self.buttonPressedTime > self.buttonPressTime then
					if not self.openColorConfiguratorDialog then
						self.customColorButton:raiseCallback("onClickCallback", _, self.pressedButton)
						self.openColorConfiguratorDialog = true
					end
				end
			else
				self.buttonPressedTime = 0
				self.openColorConfiguratorDialog = false
				self.pressedButton = nil
			end
		end
	end
end

function ColorConfiguratorPickerDialog:onClickRemoveColorButton(element)
	local colorIndex = self.additionalColorsMapping[element]
	local selectedColor = self.colors[colorIndex]

	if selectedColor ~= nil and selectedColor.isAddConfig then
		g_gui:showYesNoDialog({
			text = string.format(g_i18n:getText("colorConfigurator_ui_removeColor"), selectedColor.name),
			callback = function (yes)
				if yes then
					local filename = self.target.storeItem.xmlFilename
					local configName = self.args.configName

					self.colorConfigurator:removeAdditionalConfig(filename, configName, selectedColor)
					self:setColors(unpack(self.previousColors))
				end
			end
		})
	end
end

function ColorConfiguratorPickerDialog:updateBlurArea()
	local gui = g_gui.guis[self.name]

	if gui.blurAreaActive then
		local x, y, width, height = self:getBlurArea()

		if x ~= nil then
			g_depthOfFieldManager:popArea()
			g_depthOfFieldManager:pushArea(x, y, width, height)
		end
	end
end